/*
  Copyright 1996-2003
  Simon Whiteside

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  $Id: XMLElementObjectEnumerator.java,v 1.9 2003/03/10 19:37:28 simkin_cvs Exp $
*/
package simkin;
import org.apache.crimson.tree.*;
import org.xml.sax.*;
import org.w3c.dom.*;
import java.util.*;

/**
 * This class enumerates the elements in an XMLElementObject 
 * 
 *
*/
public class XMLElementObjectEnumerator implements ExecutableIterator{
    XMLElementObject m_ElementObject;
    String m_TagName="";
    int m_NodeNum=0;
    /**
     * Constructor - enumerates over every child node of this object
     */
    public XMLElementObjectEnumerator(XMLElementObject elementObject){
	m_ElementObject=elementObject;
	// first wind forward to the first occurrence of the tag
	findNextNode();
    }
    /**
     * Constructor - enumerates over children of this object with the matching tag
     */
    public XMLElementObjectEnumerator(XMLElementObject elementObject,String tagName){
	m_ElementObject=elementObject;
	m_TagName=tagName;
	// first wind forward to the first occurrence of the tag
	findNextNode();
    }
    /**
     * This method resets the enumeration to the beginning - you can now iterate over the list again
     */
    public void reset(){
	m_NodeNum=0;
	findNextNode();
    }
    /**
     * returns the next object in the enumeration - or null if there are no more
     */
    public Object next(){
	XMLElementObject ret=null;
	NodeList nodes=m_ElementObject.getElement().getChildNodes();
	if (m_NodeNum<nodes.getLength()){
	    Element elem=(Element)nodes.item(m_NodeNum);
	    String name=elem.getTagName();
	    ret=m_ElementObject.createXMLElementObject(Utils.addStrings(m_ElementObject.getLocation(),".",name),elem);
	    m_NodeNum++;
	    findNextNode();
	}
	return ret;
    }
    /**
     * this method winds m_NodeNum onto the next matching element - or beyond the end of the list for the end of enumeration
     */
    void findNextNode(){
      NodeList nodes=m_ElementObject.getElement().getChildNodes();
      for (;m_NodeNum<nodes.getLength();m_NodeNum++){
	Node node=nodes.item(m_NodeNum);
	if (node.getNodeType()==Node.ELEMENT_NODE && (m_TagName.length()==0 || node.getNodeName().equals(m_TagName))){
	  break;
	}
      }
    }
}
